<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Applicator;

use Dedi\SyliusVoucherPlugin\Exception\ChannelMismatchException;
use Dedi\SyliusVoucherPlugin\Exception\VoucherNotFoundException;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Doctrine\Persistence\ObjectManager;
use RuntimeException;
use Sylius\Component\Order\Processor\OrderProcessorInterface;

final class VoucherApplicator implements VoucherApplicatorInterface
{
    private readonly VoucherRepositoryInterface $voucherRepository;

    private readonly OrderProcessorInterface $orderProcessor;

    private readonly ObjectManager $orderManager;

    public function __construct(
        VoucherRepositoryInterface $voucherRepository,
        OrderProcessorInterface $orderProcessor,
        ObjectManager $orderManager,
    ) {
        $this->voucherRepository = $voucherRepository;
        $this->orderProcessor = $orderProcessor;
        $this->orderManager = $orderManager;
    }

    /**
     * @param string|VoucherInterface|mixed $voucher
     */
    public function apply(OrderInterface $order, $voucher): void
    {
        if (is_string($voucher)) {
            $voucher = $this->getVoucher($voucher);
        }

        if (!$voucher instanceof VoucherInterface) {
            throw new VoucherNotFoundException($voucher);
        }

        $orderChannel = $order->getChannel();
        if (null === $orderChannel) {
            throw new RuntimeException('The channel on the order cannot be null');
        }

        $voucherChannel = $voucher->getChannel();
        if (null === $voucherChannel) {
            throw new RuntimeException('The channel on the voucher cannot be null');
        }

        if ($orderChannel !== $voucherChannel) {
            throw new ChannelMismatchException($voucherChannel, $orderChannel);
        }

        $order->addVoucher($voucher);

        $this->orderProcessor->process($order);

        $this->orderManager->flush();
    }

    /**
     * @param string|VoucherInterface|mixed $voucher
     */
    public function remove(OrderInterface $order, $voucher): void
    {
        if (is_string($voucher)) {
            $voucher = $this->getVoucher($voucher);
        }

        if (!$voucher instanceof VoucherInterface) {
            throw new VoucherNotFoundException($voucher);
        }

        $order->removeVoucher($voucher);

        $this->orderProcessor->process($order);

        $this->orderManager->flush();
    }

    private function getVoucher(string $voucherCode): VoucherInterface
    {
        $voucher = $this->voucherRepository->findOneByCode($voucherCode);

        if (null === $voucher) {
            throw new VoucherNotFoundException($voucherCode);
        }

        return $voucher;
    }
}
