<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\DependencyInjection;

use Dedi\SyliusVoucherPlugin\Doctrine\ORM\VoucherRepository;
use Dedi\SyliusVoucherPlugin\Form\Type\VoucherChannelConfigurationType;
use Dedi\SyliusVoucherPlugin\Form\Type\VoucherConfigurationImageType;
use Dedi\SyliusVoucherPlugin\Form\Type\VoucherConfigurationType;
use Dedi\SyliusVoucherPlugin\Form\Type\VoucherType;
use Dedi\SyliusVoucherPlugin\Model\Voucher;
use Dedi\SyliusVoucherPlugin\Model\VoucherChannelConfiguration;
use Dedi\SyliusVoucherPlugin\Model\VoucherChannelConfigurationInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfiguration;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationImage;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationImageInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Bundle\ResourceBundle\Controller\ResourceController;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Bundle\ResourceBundle\SyliusResourceBundle;
use Sylius\Component\Resource\Factory\Factory;
use Symfony\Component\Config\Definition\Builder\ArrayNodeDefinition;
use Symfony\Component\Config\Definition\Builder\NodeBuilder;
use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    public function getConfigTreeBuilder(): TreeBuilder
    {
        if (method_exists(TreeBuilder::class, 'getRootNode')) {
            $treeBuilder = new TreeBuilder('dedi_sylius_voucher');
            $rootNode = $treeBuilder->getRootNode();
        } else {
            // BC layer for symfony/config 4.1 and older
            $treeBuilder = new TreeBuilder();
            $rootNode = $treeBuilder->root('dedi_sylius_voucher');
        }

        $rootNode
            ->addDefaultsIfNotSet()
            ->children()
                ->scalarNode('driver')->defaultValue(SyliusResourceBundle::DRIVER_DOCTRINE_ORM)->end()
                ->integerNode('code_length')
                    ->defaultValue(20)
                    ->info('The length of the generated voucher code')
                    ->min(1)
                    ->max(255)
                    ->example(16)
                ->end()
            ->end()
            ->children()
                ->scalarNode('use_no_restriction_rule')
                    ->defaultValue(true)
                    ->info('Define if voucher can be used without restriction in the checkout process.')
                    ->example(true)
                ->end()
            ->end()
        ;

        $this->addResourcesSection($rootNode);

        return $treeBuilder;
    }

    private function addResourcesSection(ArrayNodeDefinition $node): void
    {
        $resourcesNode = $node
            ->children()
                ->arrayNode('resources')
                    ->addDefaultsIfNotSet()
                    ->children()
        ;

        $this->addVoucherSection($resourcesNode);
        $this->addVoucherConfigurationSection($resourcesNode);
        $this->addVoucherConfigurationImageSection($resourcesNode);
        $this->addChannelConfigurationSection($resourcesNode);

        $resourcesNode
                    ->end()
                ->end()
            ->end()
        ;
    }

    private function addVoucherSection(NodeBuilder $nodeBuilder): void
    {
        $nodeBuilder
            ->arrayNode('voucher')
                ->addDefaultsIfNotSet()
                ->children()
                    ->variableNode('options')->end()
                    ->arrayNode('classes')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->scalarNode('model')->defaultValue(Voucher::class)->cannotBeEmpty()->end()
                            ->scalarNode('interface')->defaultValue(VoucherInterface::class)->cannotBeEmpty()->end()
                            ->scalarNode('controller')->defaultValue(ResourceController::class)->cannotBeEmpty()->end()
                            ->scalarNode('repository')->defaultValue(VoucherRepository::class)->cannotBeEmpty()->end()
                            ->scalarNode('form')->defaultValue(VoucherType::class)->end()
                            ->scalarNode('factory')->defaultValue(Factory::class)->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;
    }

    private function addVoucherConfigurationSection(NodeBuilder $nodeBuilder): void
    {
        $nodeBuilder
            ->arrayNode('voucher_configuration')
                ->addDefaultsIfNotSet()
                ->children()
                    ->variableNode('options')->end()
                    ->arrayNode('classes')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->scalarNode('model')->defaultValue(VoucherConfiguration::class)->cannotBeEmpty()->end()
                            ->scalarNode('interface')->defaultValue(VoucherConfigurationInterface::class)->cannotBeEmpty()->end()
                            ->scalarNode('controller')->defaultValue(ResourceController::class)->cannotBeEmpty()->end()
                            ->scalarNode('repository')->defaultValue(EntityRepository::class)->cannotBeEmpty()->end()
                            ->scalarNode('form')->defaultValue(VoucherConfigurationType::class)->end()
                            ->scalarNode('factory')->defaultValue(Factory::class)->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;
    }

    private function addVoucherConfigurationImageSection(NodeBuilder $nodeBuilder): void
    {
        $nodeBuilder
            ->arrayNode('voucher_configuration_image')
                ->addDefaultsIfNotSet()
                ->children()
                    ->variableNode('options')->end()
                    ->arrayNode('classes')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->scalarNode('model')->defaultValue(VoucherConfigurationImage::class)->cannotBeEmpty()->end()
                            ->scalarNode('interface')->defaultValue(VoucherConfigurationImageInterface::class)->cannotBeEmpty()->end()
                            ->scalarNode('controller')->defaultValue(ResourceController::class)->cannotBeEmpty()->end()
                            ->scalarNode('repository')->defaultValue(EntityRepository::class)->cannotBeEmpty()->end()
                            ->scalarNode('form')->defaultValue(VoucherConfigurationImageType::class)->end()
                            ->scalarNode('factory')->defaultValue(Factory::class)->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;
    }

    private function addChannelConfigurationSection(NodeBuilder $nodeBuilder): void
    {
        $nodeBuilder
            ->arrayNode('voucher_channel_configuration')
                ->addDefaultsIfNotSet()
                ->children()
                    ->variableNode('options')->end()
                    ->arrayNode('classes')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->scalarNode('model')->defaultValue(VoucherChannelConfiguration::class)->cannotBeEmpty()->end()
                            ->scalarNode('interface')->defaultValue(VoucherChannelConfigurationInterface::class)->cannotBeEmpty()->end()
                            ->scalarNode('controller')->defaultValue(ResourceController::class)->cannotBeEmpty()->end()
                            ->scalarNode('repository')->defaultValue(EntityRepository::class)->cannotBeEmpty()->end()
                            ->scalarNode('form')->defaultValue(VoucherChannelConfigurationType::class)->end()
                            ->scalarNode('factory')->defaultValue(Factory::class)->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ;
    }
}
