<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Fixture\Factory;

use DateTime;
use Dedi\SyliusVoucherPlugin\Generator\VoucherCodeGeneratorInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Faker\Factory;
use Faker\Generator;
use Sylius\Bundle\CoreBundle\Fixture\Factory\AbstractExampleFactory;
use Sylius\Bundle\CoreBundle\Fixture\Factory\ExampleFactoryInterface;
use Sylius\Bundle\CoreBundle\Fixture\OptionsResolver\LazyOption;
use Sylius\Component\Channel\Repository\ChannelRepositoryInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Currency\Model\CurrencyInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\OptionsResolver\Options;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Webmozart\Assert\Assert;

class VoucherExampleFactory extends AbstractExampleFactory implements ExampleFactoryInterface
{
    protected VoucherRepositoryInterface $voucherRepository;

    protected FactoryInterface $voucherFactory;

    protected VoucherCodeGeneratorInterface $voucherCodeGenerator;

    protected ChannelRepositoryInterface $channelRepository;

    protected RepositoryInterface $currencyRepository;

    protected Generator $faker;

    protected OptionsResolver $optionsResolver;

    public function __construct(
        VoucherRepositoryInterface $voucherRepository,
        FactoryInterface $voucherFactory,
        VoucherCodeGeneratorInterface $voucherCodeGenerator,
        ChannelRepositoryInterface $channelRepository,
        RepositoryInterface $currencyRepository,
    ) {
        $this->voucherRepository = $voucherRepository;
        $this->voucherFactory = $voucherFactory;
        $this->voucherCodeGenerator = $voucherCodeGenerator;
        $this->channelRepository = $channelRepository;
        $this->currencyRepository = $currencyRepository;

        $this->faker = Factory::create();
        $this->optionsResolver = new OptionsResolver();

        $this->configureOptions($this->optionsResolver);
    }

    public function create(array $options = []): VoucherInterface
    {
        $options = $this->optionsResolver->resolve($options);

        return $this->createVoucher($options);
    }

    protected function createVoucher(array $options): VoucherInterface
    {
        /** @var VoucherInterface|null $voucher */
        $voucher = $this->voucherRepository->findOneBy(['code' => $options['code']]);
        if (null === $voucher) {
            /** @var VoucherInterface $voucher */
            $voucher = $this->voucherFactory->createNew();
        }

        /** @var CurrencyInterface $currency */
        $currency = $options['currency'];

        $voucher->setCode($options['code']);
        $voucher->setChannel($options['channel']);
        $voucher->setCurrencyCode($currency->getCode());

        if (null === $voucher->getId()) {
            // We can change initial amount only if it wasn't specified before
            $voucher->setInitialAmount($options['initial_amount']);
        }

        if (null !== $options['amount']) {
            $voucher->setAmount($options['amount']);
        }

        $voucher->setEnabled($options['enabled']);

        if (null !== $options['date_end']) {
            $voucher->setDateEnd(new DateTime($options['date_end']));
        }

        return $voucher;
    }

    protected function configureOptions(OptionsResolver $resolver): void
    {
        $resolver
            ->setDefault('date_end', null)
            ->setAllowedTypes('date_end', ['null', 'string'])

            ->setDefault('code', fn (Options $options): string => $this->voucherCodeGenerator->generate())

            ->setDefault('channel', LazyOption::randomOne($this->channelRepository))
            ->setAllowedTypes('channel', ['null', 'string', ChannelInterface::class])
            ->setNormalizer('channel', LazyOption::findOneBy($this->channelRepository, 'code'))

            ->setDefault('currency', function (Options $options): CurrencyInterface {
                /** @var ChannelInterface|mixed $channel */
                $channel = $options['channel'];
                Assert::isInstanceOf($channel, ChannelInterface::class);

                $currency = $channel->getBaseCurrency();
                Assert::notNull($currency);

                return $currency;
            })
            ->setAllowedTypes('currency', ['null', 'string', CurrencyInterface::class])
            ->setNormalizer('currency', function (Options $options, $currencyCode): CurrencyInterface {
                if ($currencyCode instanceof CurrencyInterface) {
                    $currency = $currencyCode;
                    $currencyCode = $currency->getCode();
                } else {
                    /** @var CurrencyInterface|null $currency */
                    $currency = $this->currencyRepository->findOneBy(['code' => $currencyCode]);
                }

                /** @var ChannelInterface|mixed $channel */
                $channel = $options['channel'];
                $channelCurrenciesCodes = $channel->getCurrencies()->map(function (CurrencyInterface $currency): string {
                    $currencyCode = $currency->getCode();
                    Assert::notNull($currencyCode);

                    return $currencyCode;
                })->toArray();

                Assert::notNull($currency, sprintf(
                    'Currency %s was not found. Use one of: %s',
                    $currencyCode,
                    implode(', ', $channelCurrenciesCodes),
                ));

                Assert::isInstanceOf($channel, ChannelInterface::class);

                Assert::oneOf($currency, $channel->getCurrencies()->toArray(), sprintf(
                    'Expecting one of %s currencies, got: %s',
                    implode(', ', $channelCurrenciesCodes),
                    $currencyCode,
                ));

                return $currency;
            })

            ->setDefault('initial_amount', fn (Options $options): float => $this->faker->randomFloat(2, 0.01, 1000.00))
            ->setAllowedTypes('initial_amount', ['null', 'float', 'int'])
            ->setNormalizer('initial_amount', fn (Options $options, float $initialAmount): int => (int) $initialAmount * 100)

            ->setDefault('amount', null)
            ->setAllowedTypes('amount', ['null', 'float', 'int'])
            ->setNormalizer('amount', function (Options $options, ?float $amount): ?int {
                if (null === $amount) {
                    return null;
                }

                $amount = (int) $amount * 100;

                Assert::lessThanEq($amount, $options['initial_amount']);

                return $amount;
            })

            ->setDefault('enabled', true)
            ->setAllowedTypes('enabled', 'bool')
        ;
    }
}
