<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Form\Type;

use Dedi\SyliusVoucherPlugin\Checker\VoucherCheckerContextInterface;
use Dedi\SyliusVoucherPlugin\Controller\Action\AddVoucherToOrderCommand;
use Dedi\SyliusVoucherPlugin\Model\Voucher;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Sylius\Bundle\MoneyBundle\Templating\Helper\ConvertMoneyHelperInterface;
use Sylius\Bundle\MoneyBundle\Templating\Helper\FormatMoneyHelperInterface;
use Sylius\Component\Core\Context\ShopperContextInterface;
use Sylius\Component\Core\Model\ShopUser;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

final class AddVoucherToOrderType extends AbstractType
{
    private readonly array $validationGroups;

    private readonly TokenStorageInterface $tokenStorage;

    private readonly VoucherCheckerContextInterface $checkerContext;

    private readonly VoucherRepositoryInterface $voucherRepository;

    private readonly FormatMoneyHelperInterface $formatMoneyHelper;

    private readonly ConvertMoneyHelperInterface $convertMoneyHelper;

    private readonly ShopperContextInterface $shopperContext;

    public function __construct(
        array $validationGroups,
        TokenStorageInterface $tokenStorage,
        VoucherCheckerContextInterface $checkerContext,
        VoucherRepositoryInterface $voucherRepository,
        FormatMoneyHelperInterface $formatMoneyHelper,
        ConvertMoneyHelperInterface $convertMoneyHelper,
        ShopperContextInterface $shopperContext,
    ) {
        $this->validationGroups = $validationGroups;
        $this->tokenStorage = $tokenStorage;
        $this->checkerContext = $checkerContext;
        $this->voucherRepository = $voucherRepository;
        $this->formatMoneyHelper = $formatMoneyHelper;
        $this->convertMoneyHelper = $convertMoneyHelper;
        $this->shopperContext = $shopperContext;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $user = $this->tokenStorage->getToken()->getUser();

        if (!$user instanceof ShopUser) {
            throw new AccessDeniedException('This user does not have access to this section.');
        }

        $builder
            ->add('voucher', EntityType::class, [
                'class' => Voucher::class,
                'multiple' => false,
                'expanded' => true,
                'choices' => $this->checkerContext->getVouchersEligible(
                    $this->voucherRepository->findEnabledByUser($user->getCustomer()),
                ),
                'choice_label' => fn (Voucher $voucher): string => sprintf(
                    '%s - %s',
                    $voucher->getCode(),
                    $this->formatMoneyHelper->formatAmount(
                        (int) $this->convertMoneyHelper->convertAmount(
                            $voucher->getAmount(),
                            $this->shopperContext->getChannel()->getCode(),
                            $this->shopperContext->getCurrencyCode(),
                        ),
                        $this->shopperContext->getCurrencyCode(),
                        $this->shopperContext->getLocaleCode(),
                    ),
                ),
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => AddVoucherToOrderCommand::class,
            'validation_groups' => $this->validationGroups,
        ]);
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_voucher_add_voucher_to_order';
    }
}
